@@ -2,7 +2,7 @@
 #
 #  Net::Server::PreFork - Net::Server personality
 #  
-#  $Id: PreFork.pm,v 1.5 2001/08/23 17:02:57 rhandom Exp $
+#  $Id: PreFork.pm,v 1.8 2001/08/24 17:52:36 rhandom Exp $
 #  
 #  Copyright (C) 2001, Paul T Seamons
 #                      paul@seamons.com
@@ -22,15 +22,14 @@ package Net::Server::PreFork;
 use strict;
 use vars qw($VERSION @ISA $LOCK_EX $LOCK_UN);
 use POSIX qw(WNOHANG);
-use Fcntl ();
-use Net::Server;
+use Net::Server::PreForkSimple;
 use Net::Server::SIG qw(register_sig check_sigs);
 use IO::Select ();
 
 $VERSION = $Net::Server::VERSION; # done until separated
 
 ### fall back to parent methods
-@ISA = qw(Net::Server);
+@ISA = qw(Net::Server::PreForkSimple);
 
 
 ### override-able options for this package
@@ -41,11 +40,11 @@ sub options {
 
   $self->SUPER::options($ref);
 
-  foreach ( qw(min_servers max_servers
+  foreach ( qw(min_servers
                min_spare_servers max_spare_servers
-               spare_servers max_requests max_dequeue
-               check_for_dead check_for_waiting check_for_dequeue
-               lock_file serialize) ){
+               spare_servers
+               check_for_waiting
+               ) ){
     $prop->{$_} = undef unless exists $prop->{$_};
     $ref->{$_} = \$prop->{$_};
   }
@@ -75,12 +74,9 @@ information.
 
   ### some default values to check for
   my $d = {min_servers       => 5,    # min num of servers to always have running
-           max_servers       => 50,   # max num of servers to run
            min_spare_servers => 2,    # min num of servers just sitting there
            max_spare_servers => 10,   # max num of servers just sitting there
-           max_requests      => 1000, # num of requests for each child to handle
-           check_for_waiting => 10,   # how often to see we have to many servers
-           check_for_dead    => 30,   # how often to see if children are alive
+           check_for_waiting => 10,   # how often to see if children laying around
            };
   foreach (keys %$d){
     $prop->{$_} = $d->{$_}
@@ -102,66 +98,9 @@ information.
                  ."\"max_servers\"");
   }
 
-  ### we will need to force a select handle
-  $prop->{multi_port} = 1;
-
-  ### I need to know who is the parent
-  $prop->{ppid} = $$;
-
 }
 
 
-### now that we are bound, prepare serialization
-sub post_bind {
-  my $self = shift;
-  my $prop = $self->{server};
-
-  ### do the parents
-  $self->SUPER::post_bind;
-
-  ### clean up method to use for serialization
-  if( ! defined($prop->{serialize}) 
-      || $prop->{serialize} !~ /^(flock|semaphore|pipe)$/ ){
-    $prop->{serialize} = 'flock';
-  }
-
-  ### set up lock file
-  if( $prop->{serialize} eq 'flock' ){
-    $self->log(3,"Setting up serialization via flock");
-    if( defined($prop->{lock_file}) ){
-      $prop->{lock_file_unlink} = undef;
-    }else{
-      $prop->{lock_file} = POSIX::tmpnam();
-      $prop->{lock_file_unlink} = 1;
-    }
-
-  ### set up semaphore
-  }elsif( $prop->{serialize} eq 'semaphore' ){
-    $self->log(3,"Setting up serialization via semaphore");
-    require "IPC/SysV.pm";
-    require "IPC/Semaphore.pm";
-    my $s = IPC::Semaphore->new(IPC::SysV::IPC_PRIVATE(),
-                                1,
-                                IPC::SysV::S_IRWXU() | IPC::SysV::IPC_CREAT(),
-                                ) || $self->fatal("Semaphore error [$!]");
-    $s->setall(1) || $self->fatal("Semaphore create error [$!]");
-    $prop->{sem} = $s;
-    
-  ### set up pipe
-  }elsif( $prop->{serialize} eq 'pipe' ){
-    pipe( _WAITING, _READY );
-    _READY->autoflush(1);
-    _WAITING->autoflush(1);
-    $prop->{_READY}   = *_READY;
-    $prop->{_WAITING} = *_WAITING;
-    print _READY "First\n";
-    
-  }else{
-    $self->fatal("Unknown serialization type \"$prop->{serialize}\"");
-  }
-
-}
-
 ### prepare for connections
 sub loop {
   my $self = shift;
@@ -292,73 +231,6 @@ sub run_child {
 }
 
 
-### hooks at the beginning and end of forked child processes
-sub child_init_hook {}
-sub child_finish_hook {}
-
-
-### We can only let one process do the selecting at a time
-### this override makes sure that nobody else can do it
-### while we are.  We do this either by opening a lock file
-### and getting an exclusive lock (this will block all others
-### until we release it) or by using semaphores to block
-sub accept {
-  my $self = shift;
-  my $prop = $self->{server};
-
-  local *LOCK;
-
-  ### serialize the child accepts
-  if( $prop->{serialize} eq 'flock' ){
-    open(LOCK,">$prop->{lock_file}")
-      || $self->fatal("Couldn't open lock file \"$prop->{lock_file}\" [$!]");
-    flock(LOCK,Fcntl::LOCK_EX())
-      || $self->fatal("Couldn't get lock on file \"$prop->{lock_file}\" [$!]");
-
-  }elsif( $prop->{serialize} eq 'semaphore' ){
-    $prop->{sem}->op( 0, -1, IPC::SysV::SEM_UNDO() )
-      || $self->fatal("Semaphore Error [$!]");
-
-  }elsif( $prop->{serialize} eq 'pipe' ){
-    scalar <_WAITING>; # read one line - kernel says who gets it
-  }
-
-
-  ### now do the accept method
-  my $accept_val = $self->SUPER::accept();
-
-
-  ### unblock serialization
-  if( $prop->{serialize} eq 'flock' ){
-    flock(LOCK,Fcntl::LOCK_UN());
-
-  }elsif( $prop->{serialize} eq 'semaphore' ){
-    $prop->{sem}->op( 0, 1, IPC::SysV::SEM_UNDO() )
-      || $self->fatal("Semaphore Error [$!]");
-
-  }elsif( $prop->{serialize} eq 'pipe' ){
-    print _READY "Next!\n";
-  }
-
-  ### return our success
-  return $accept_val;
-
-}
-
-
-### is the looping done (non zero value says its done)
-sub done {
-  my $self = shift;
-  my $prop = $self->{server};
-  return 1 if $prop->{requests} >= $prop->{max_requests};
-  return 1 if $prop->{SigHUPed};
-  if( ! kill(0,$prop->{ppid}) ){
-    $self->log(3,"Parent process gone away. Shutting down");
-    return 1;
-  }
-}
-
-
 ### now the parent will wait for the kids
 sub run_parent {
   my $self=shift;
@@ -459,9 +331,6 @@ sub run_parent {
 }
 
 
-### allow for other process to tie in to the parent read
-sub parent_read_hook {}
-
 ### routine to determine if more children need to be started or stopped
 sub coordinate_children {
   my $self = shift;
@@ -554,27 +423,6 @@ sub coordinate_children {
 }
 
 
-### routine to shut down the server (and all forked children)
-sub server_close {
-  my $self = shift;
-  my $prop = $self->{server};
-
-  ### if a parent, fork off cleanup sub and close
-  if( ! defined $prop->{ppid} || $prop->{ppid} == $$ ){
-
-    $self->SUPER::server_close();
-
-  ### if a child, signal the parent and close
-  ### normally the child shouldn't, but if they do...
-  }else{
-
-    kill(2,$prop->{ppid});
-
-  }
-  
-  exit;
-}
-
 1;
 
 __END__
@@ -596,9 +444,10 @@ Net::Server::PreFork - Net::Server personality
 
 =head1 DESCRIPTION
 
-Please read the pod on Net::Server first.  This module
-is a personality, or extension, or sub class, of the
-Net::Server module.
+Please read the pod on Net::Server and Net::Server::PreForkSimple
+first.  This module is a personality, or extension, or sub class,
+of the Net::Server::PreForkSimple class which is a sub class of
+Net::Server.  See L<Net::Server::PreForkSimple>.
 
 This personality binds to one or more ports and then forks
 C<min_servers> child process.  The server will make sure
@@ -616,8 +465,10 @@ Please see the sample listed in Net::Server.
 =head1 COMMAND LINE ARGUMENTS
 
 In addition to the command line arguments of the Net::Server
-base class, Net::Server::PreFork contains several other 
-configurable parameters.
+base class and the Net::Server::PreForkSimple parent class,
+Net::Server::PreFork contains several other configurable
+parameters.  You really should also see
+L<Net::Server::PreForkSimple>.
 
   Key               Value                   Default
   min_servers       \d+                     5
@@ -659,58 +510,11 @@ See I<min_spare_servers>.
 The maximum number of child servers to start.  This does not
 apply to dequeue processes.
 
-=item max_requests
-
-The number of client connections to receive before a
-child terminates.
-
-=item serialize
-
-Determines whether the server serializes child connections.
-Options are undef, flock, semaphore, or pipe.  Default is undef.
-On multi_port servers or on servers running on Solaris, the
-default is flock.  The flock option uses blocking exclusive
-flock on the file specified in I<lock_file> (see below).
-The semaphore option uses IPC::Semaphore (thanks to Bennett
-Todd) for giving some sample code.  The pipe option reads on a 
-pipe to choose the next.  the flock option should be the
-most bulletproof while the pipe option should be the most
-portable.  (Flock is able to reliquish the block if the
-process dies between accept on the socket and reading
-of the client connection - semaphore and pipe do not)
-
-=item lock_file
-
-Filename to use in flock serialized accept in order to
-serialize the accept sequece between the children.  This
-will default to a generated temporary filename.  If default
-value is used the lock_file will be removed when the server
-closes.
-
-=item check_for_dead
-
-Seconds to wait before checking to see if a child died
-without letting the parent know.
-
 =item check_for_waiting
 
 Seconds to wait before checking to see if we can kill
 off some waiting servers.
 
-=item max_dequeue
-
-The maximum number of dequeue processes to start.  If a
-value of zero or undef is given, no dequeue processes will
-be started.  The number of running dequeue processes will
-be checked by the check_for_dead variable.
-
-=item check_for_dequeue
-
-Seconds to wait before forking off a dequeue process.  It
-is intended to use the dequeue process to take care of 
-items such as mail queues.  If a value of undef is given,
-no dequeue processes will be started.
-
 =back
 
 =head1 CONFIGURATION FILE
@@ -764,30 +568,16 @@ this point C<min_servers> are forked and wait for
 connections.  When a child accepts a connection, finishs
 processing a client, or exits, it relays that information to
 the parent, which keeps track and makes sure there are
-enough children to fulfill C<min_servers>, C<spare_servers>,
-and C<max_servers>.
+enough children to fulfill C<min_servers>, C<min_spare_servers>,
+C<max_spare_servers>, and C<max_servers>.
 
 =head1 HOOKS
 
-There are three additional hooks in the PreFork server.
+There is one additional hook in the PreFork server.
+See L<Net::Server::PreForkSimple> for other hooks.
 
 =over 4
 
-=item C<$self-E<gt>child_init_hook()>
-
-This hook takes place immeditately after the child process
-forks from the parent and before the child begins
-accepting connections.  It is intended for any addiotional
-chrooting or other security measures.  It is suggested
-that all perl modules be used by this point, so that
-the most shared memory possible is used.
-
-=item C<$self-E<gt>child_finish_hook()>
-
-This hook takes place immediately before the child tells
-the parent that it is exiting.  It is intended for 
-saving out logged information or other general cleanup.
-
 =item C<$self-E<gt>parent_read_hook()>
 
 This hook occurs any time that the parent reads information
@@ -800,18 +590,6 @@ argument.
 
 See L<Net::Server>
 
-=head1 FILES
-
-  The following files are installed as part of this
-  distribution.
-
-  Net/Server.pm
-  Net/Server/Fork.pm
-  Net/Server/INET.pm
-  Net/Server/MultiType.pm
-  Net/Server/PreFork.pm
-  Net/Server/Single.pm
-
 =head1 AUTHOR
 
 Paul T. Seamons paul@seamons.com
@@ -825,9 +603,12 @@ See L<Net::Server>
 Please see also
 L<Net::Server::Fork>,
 L<Net::Server::INET>,
-L<Net::Server::PreFork>,
+L<Net::Server::PreForkSimple>,
 L<Net::Server::MultiType>,
 L<Net::Server::Single>
+L<Net::Server::SIG>
+L<Net::Server::Daemonize>
+L<Net::Server::Proto>
 
 =cut
 
